/*
 * Copyright (c), Recep Aslantas.
 *
 * MIT License (MIT), http://opensource.org/licenses/MIT
 * Full license can be found in the LICENSE file
 */

#ifndef cglm_structs_h
#define cglm_structs_h
#ifdef __cplusplus
extern "C" {
#endif

#include "cglm.h"
#include "types-struct.h"
#include "struct/vec2.h"
#include "struct/vec3.h"
#include "struct/vec4.h"
#include "struct/mat2.h"
#include "struct/mat3.h"
#include "struct/mat4.h"
#include "struct/affine.h"
#include "struct/frustum.h"
#include "struct/plane.h"
#include "struct/box.h"
#include "struct/color.h"
#include "struct/io.h"
#include "struct/cam.h"
#include "struct/quat.h"
#include "struct/euler.h"
#include "struct/project.h"
#include "struct/sphere.h"
#include "struct/curve.h"
#include "struct/affine2d.h"

#ifdef __cplusplus
}
#endif
#endif /* cglm_structs_h */
